# Application of robust multivariate correlation constancy test on stock returns
# created on April 13, 2017
# last Edit on JUne 21, 2017
rm(list=ls())
# Specify Path 
if (.Platform$OS.type == "unix") {
 # Mac
 path <- c("/Users/fangduan/Dropbox/Dissertation/Correlation Test/R")
} else if (.Platform$OS.type == "windows"){
 # Windows
 path <- c("C:/Users/Win7Adm/Dropbox/Dissertation/Correlation Test/R")
}
setwd(path)

# Define functions
argmaxest <- function(X)
{
  n <- length(X)
  vektor <- rep(0,n)
  Vari <- rep(0,n)
  varn <- var(X)
  Vari <- cumsum(X^2)/(1:n) - (cumsum(X)/(1:n))^2
  ##Vari[i] <- mean((X[1:i])^2)
  vektor <- (1:n)/sqrt(n) * (Vari - varn)
  result <- which.max(abs(vektor))
  return(result)
}

cusumtest <- function(data, est=T)
{
  n <- nrow(data)
  ll <- ncol(data)
  
  if(est==T){vec.est <- apply(data,2,argmaxest)}
  for (i in 1:length(vec.est))
  {
    if (vec.est[i] >= n-1){
      vec.est[i] <- n-2
    } else if (vec.est[i] == 1){
      vec.est[i] <- 2
    }
  }
  
  clean <- matrix(NA,ncol=ll,nrow=n)
  for (ii in 1:ll){
    if (sd(data[1:vec.est[ii],ii]) * sd(data[-(1:vec.est[ii]),ii]) >0 ){
      clean[1:vec.est[ii],ii] <- data[1:vec.est[ii],ii] / sd(data[1:vec.est[ii],ii])
      clean[-(1:vec.est[ii]),ii] <- data[-(1:vec.est[ii]),ii] / sd(data[-(1:vec.est[ii]),ii])
    } else if (sd(data[1:vec.est[ii],ii]) * sd(data[-(1:vec.est[ii]),ii]) == 0){
      clean[,ii] <- data[,ii] / sd(data[,ii])
    }
  }
  
  for (j in 1:(ll-1)){
    for (k in (j+1):ll){
      lhs <- paste("x",j,k,sep="")
      rhs <- paste("clean[,",j,"]*clean[,",k,"]",sep="")
      eq <- paste(lhs,rhs,sep=" <- ")
      eval(parse(text=eq))
    }
  }
  
  x <- cbind(x12,x13,x14,x15,x16,x17,x18,
             x23,x24,x25,x26,x27,x28,
             x34,x35,x36,x37,x38,
             x45,x46,x47,x48,
             x56,x57,x58,
             x67,x68,
             x78)
  Omega <- cov(x)
  
  S <- apply(x,2,function(x) cumsum(x)-(1:n)/n * sum(x))
  process <- sapply(1:nrow(S),function(s) 1/sqrt(n) * sqrt( S[s,] %*% solve(Omega) %*% as.matrix(S[s,])) )
  
  result <- abs(process)
  
  return(list(stat = max(result),cp = vec.est,proc = result,clean=x))
}

block.boot <- function(data,B,vec_est = c(0,0,0,0,0,0,0,0), est = T){
  n <- dim(data)[1]
  d <- dim(data)[2]
  
  
  if (est == T){vec_est <- apply(data,2,argmaxest)}
  
  sd1 <- rep(NA,d)
  sd2 <- rep(NA,d)
  for (jj in 1:d){
    sd1[jj] <- sd(data[1:vec_est[jj],jj])
    sd2[jj] <- sd(data[-(1:vec_est[jj]),jj])
  }
  
  dataDemean <- apply(data,2,function(x) x-mean(x))
  T <- length(data[,1])
  l <- floor(T^(1/3))
  anzbl <- floor(T/l)
  if(l * anzbl < T) anzbl <- anzbl+1
  
  boot_dist <- rep(NA,B)
  for(b in 1:B)
  {
    print(b)
    # set.seed(b)
    index <- sample(1:anzbl,anzbl,replace=T)
    index <- sapply(index,function(x) seq(1+(x-1)*l,x*l,by=1))
    index <- as.vector(index)
    problem <- which(index > T)
    if(length(problem) > 0) index[problem] <- index[problem]-T
    xboot <- dataDemean[index,]
    
    for (kk in 1:d)
    {
      if(sd(xboot[1:vec_est[kk],kk]) * sd(xboot[-(1:vec_est[kk]),kk]) > 0)
      {xboot[1:vec_est[kk],kk] <- xboot[1:vec_est[kk],kk] / sd(xboot[1:vec_est[kk],kk]) * sd1[kk]
      xboot[-(1:vec_est[kk]),kk] <- xboot[-(1:vec_est[kk]),kk] / sd(xboot[-(1:vec_est[kk]),kk]) * sd2[kk]
      }
      if(sd(xboot[1:vec_est[kk],kk]) * sd(xboot[-(1:vec_est[kk]),kk]) == 0)
      {xboot[,kk] <- xboot[,kk] / sd(xboot[,kk])}
    }
    boot_dist[b] = cusumtest(xboot)$stat
  }
  return(boot_dist)
}

#### Load data set
stoxx <- read.table(paste(path,"/data/eurostoxx50.txt",sep=""),stringsAsFactors=FALSE)
data <- cbind(stoxx$ARCELOR,stoxx$BASF,stoxx$DAIMLER,stoxx$ENEL,stoxx$INGGROEP,stoxx$INTESA,stoxx$LVMH,stoxx$SANOFI)
colnames(data) <- c("ARCELOR","BASF","DAIMLER","ENEL","INGGROEP","INTESA","LVMH","SANOFI")
data <- data[8351:9655,] # Jan 01 2005 - Jan 01 2010
mydate <- as.Date(stoxx$Date,format = "%d.%m.%Y")
mydate <- mydate[8351:9654]

# log return
res <- sapply(1:ncol(data), function(x) diff(log(data[,x]),lag = 1))
colnames(res) <- c("ARCELOR","BASF","DAIMLER","ENEL","INGGROEP","INTESA","LVMH","SANOFI")
data <- res

# plots for rolling correlations and rolling marginal variances ############
source("plots_empirical_analysis.R")

# detect variance breaks (function 'Var_Changes' allows for multiple break points)
source("Varianztest.R")
apply(data,2,Var_Changes)

# Perform Demetrescu/Wied (2017) test for constant correlation ############
test <- cusumtest(data)
B <- 10999
boot_dist <- block.boot(data,B)
pval = mean(test$stat <= boot_dist,na.rm=T)

#### Plots in emprical analysis ############
pdf("hist.pdf",width=14)
par(cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2) + 0.1,lwd=2)
hist(unlist(boot_dist),breaks=200,xlab="",ylab="Density",main="",probability = T,col="lightblue1")
dens <- density(unlist(boot_dist))
lines(density(unlist(boot_dist)),ylim=range(dens$y),xlim=range(dens$x))
abline(v = quantile(boot_dist,prob=0.95,na.rm=T),col="blue")
dev.off()

pdf("Test_stat_process.pdf",width=14)
par(cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2) + 0.1,lwd=2)
plot(mydate,test$proc,type="l",ylim=c(0,5),xlab="Date",ylab="Test Statistics")
dev.off()

# pdf("ACF.pdf",height=14,width=14)
# par(mfrow=c(4,7),cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2)+0.1,lwd=2)
# apply(test$clean,2,acf,main="")
# dev.off()
## ACF plot with subnames for subplots
name.acf <- c("A&B","A&D","A&E","A&ING","A&INT","A&L","A&S",
              "B&D","B&E","B&ING","B&INT","B&L","B&S",
              "D&E","D&ING","D&INT","D&L","D&S",
              "E&ING","E&INT","E&L","E&S",
              "ING&INT","ING&L","ING&S",
              "INT&L","INT&S",
              "L&S")
pdf("ACF.pdf",height=14,width=14)
par(mfrow=c(4,7),cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2)+0.1,lwd=2)
for (i in 1:28){
  acf(test$clean[,i],main=name.acf[i])
}
dev.off()

# ACF plot for BASF and Daimler
pdf("ACF_BD.pdf",height=7,width=14)
par(cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2)+0.1,lwd=2)
acf(test$clean[,8],main="")
dev.off()

#### Perform Wied(2015) test ############
source("FunctionsSimulations2.R")
gen.val <- generated.values.test.bootstrap(d,B)
output <- BSWRS.CorMat(data,B,gen.val)

#### Perform Galeano and Wied (2016) test to detect the multiple break points ############
# load the NEW functions for multiple break points detection
source("new_functions_multiple_break_correlations.R")
mv_corr_changes(data)
