## Hilfsmatrix fr D1dach in Abhngigkeit von n mit Bandbreite Wurzel n und Bartlett-Kern

hilfsmatrix <- function(n)
{
hilfsmatrix <- matrix(rep(0,n^2),ncol=n)
for(i in 1:n)
 {
  for(j in 1:n)
    {
     abstand <- abs(i-j)
     hilfsmatrix[i,j] <- 1/n * max(0,1-abstand/sqrt(n))
    }
 }
return(hilfsmatrix)
}




## Test auf konstante Varianz:

testV <- function(X)
{
n <- length(X)
mx1 <- mean(X)
mx2 <- mean(X^2)
Abl <- c(1,-2*mx1)
U <- matrix(c(X^2-mx2,X-mx1),ncol=2,byrow=F)

D1dach <- t(U) %*% hilfsmatrix(n) %*% U

Ddach <- t(Abl) %*% D1dach %*% Abl
Ddach <- 1/sqrt(Ddach)
Varn <- mx2 - (mx1)^2

vektor <- rep(0,n)
Vari <- rep(0,n)
for(i in 1:n)
{
Vari[i] <- mean((X[1:i])^2) - (mean(X[1:i]))^2
##Vari[i] <- mean((X[1:i])^2)
vektor[i] <- i/sqrt(n) * (Vari[i] - Varn)
}

teststatistik <- Ddach * abs(vektor)
## teststatistik <- sum((Ddach * abs(vektor))^2)/n
return(teststatistik)
}

########################################################################################################
# The function findQuantile computes the critical values used in the algorithm.
########################################################################################################

findQuantile <- function(prob=0.95,start=1,intervall=5,steps=50,acc=50){
   interv <- c(start,start+intervall)
   for (i in 1:steps){
      mid <- mean(interv)
      if (supAbsBB(mid,acc=acc) < prob){
         interv <- c(mid,interv[2])
      } 
      else{
         interv <- c(interv[1],mid)
      }
   }
   return(interv[1])
}

########################################################################################################
# The function supAbsBB is the CDF of the supremum of the absolute value of a
# Brownian bridge.
########################################################################################################

supAbsBB <- function(x,acc=50){
   alt <- rep(c(-1,1),length.out=acc)
   return(  1 + 2 * sum(alt * exp( -2 * x^2 * (1:acc)^2)) )
}

########################################################################################################
# Procedure for detecting variance breaks
########################################################################################################


Var_Changes <- function(X){

  n <- length(X)

  a0 <- .05  # This is the initial level

  locch <- c() # This is the vector of detected changes located
  locchsort <- c() # This is the ordered vector of changes
  tpch <- c() # This is the vector of the statistics associated with locch
  tpchsort <- c() # This are the statistics associated with locchsort

  look <- 1 # This is an indicator variable to check if the algorithm should look for new changes
  iter <- 0 # Iteration number

# First, we look for changes in the correlation

  while ((look == 1) && (iter < 100)){

    iter <- iter + 1
    llocch <- length(locch)
    locch2 <- c(0,locchsort,n)

    mtp <- htp <- c()
    for (i in 1 : (llocch + 1)){
        nini <- locch2[i] + 1
        nend <- locch2[i+1]
	      tp <- rep(0,n)
        tp[nini:nend] <- testV(X[nini:nend])
        mtp[i] <- max(tp)
        htp[i] <- which.max(tp)
    }

    mmtp <- max(mtp) # Maximum of the statistics in this iteration
    hhtp <- htp[which.max(mtp)] # Estimation of the new changepoint (candidate)

    sl <- (1 - a0)^(1/(length(locch)+1))
    cv <- findQuantile(prob=sl)

    if (mmtp > cv){
       if (any(abs(locch - hhtp) < (.05 * n))){
          look = 0
       }else{
          locch <- c(locch,hhtp)
          tpch <- c(tpch,mmtp)
       }   
       sortlocch <- sort(locch,index.return=TRUE)
       locchsort <- sortlocch$x
	     tpchsort <- tpch[sortlocch$ix]
    }else{
       look <- 0
    }    

  }
  
# Then, we refine the search  
  
  if (length(locchsort) == 0){
     locchanges <- tpchanges <- c()
  }else if (length(locchsort) == 1){
     locchanges <- locchsort
     tpchanges <- tpchsort
  }else{
     ite <- 1
     while (ite == 1){   
        locchanges <- c(0,locchsort,n)
        lc <- length(locchanges) - 2
        locch <- hch <- c()
        for (i in 1 : lc){
            tp <- rep(0,n)
            nini <- locchanges[i] + 1
            nend <- locchanges[i+2]
            tp[nini:nend] <- testV(X[nini:nend])
            hch[i] <- max(tp)
            locch[i] <- which.max(tp)
        }
        mhch <- min(hch)
        wmhch <- which.min(hch)
        if (mhch < findQuantile(prob=1-a0)){
           locchsort <- locchsort[-wmhch]
        }else{
           sortlocch <- sort(locch,index.return=TRUE)
           locchanges <- sortlocch$x
	         tpchanges <- hch[sortlocch$ix]
	         ite <- 0
        }
	   }
  }
  estchanges <- locchanges / n
    
return(list("locchanges"=locchanges,"tpchanges"=tpchanges,"estchanges"=estchanges))

}

## .Call(stats:::C_pKS2, p = 1, tol = 10^(-6))