# Multiple break points detection with Binary segmentation algorithm (Galeano and Wied)
# this is new script in a compact and neat way
# created on June 18, 2017

argmaxest <- function(X)
{
  n <- length(X)
  vektor <- rep(0,n)
  Vari <- rep(0,n)
  varn <- var(X)
  Vari <- cumsum(X^2)/(1:n) - (cumsum(X)/(1:n))^2
  ##Vari[i] <- mean((X[1:i])^2)
  vektor <- (1:n)/sqrt(n) * (Vari - varn)
  result <- which.max(abs(vektor))
  return(result)
}

# this test function work for 8 dimension (suitable to the empirical analysis)
cusumtest <- function(data, vec.est=c(0,0,0,0,0,0,0,0), est=T)
{
  n <- nrow(data)
  ll <- ncol(data)
  
  if(est==T){vec.est <- apply(data,2,argmaxest)}
  
  for (i in 1:length(vec.est))
  {
    if (vec.est[i] >= n-1){
      vec.est[i] <- n-2
    } else if (vec.est[i] == 1){
      vec.est[i] <- 2
    }
  }
  
  clean <- matrix(NA,ncol=ll,nrow=n)
  for (ii in 1:ll){
    if (vec.est[ii] != 0){
      clean[1:vec.est[ii],ii] <- data[1:vec.est[ii],ii] / sd(data[1:vec.est[ii],ii])
      clean[-(1:vec.est[ii]),ii] <- data[-(1:vec.est[ii]),ii] / sd(data[-(1:vec.est[ii]),ii])
    } else if (vec.est[ii] == 0){
      clean[,ii] <- data[,ii] / sd(data[,ii])
    }
  }
  
  for (j in 1:(ll-1)){
    for (k in (j+1):ll){
      lhs <- paste("x",j,k,sep="")
      rhs <- paste("clean[,",j,"]*clean[,",k,"]",sep="")
      eq <- paste(lhs,rhs,sep=" <- ")
      eval(parse(text=eq))
    }
  }
  
  x <- cbind(x12,x13,x14,x15,x16,x17,x18,
             x23,x24,x25,x26,x27,x28,
             x34,x35,x36,x37,x38,
             x45,x46,x47,x48,
             x56,x57,x58,
             x67,x68,
             x78)
  Omega <- cov(x)
  
  S <- apply(x,2,function(x) cumsum(x)-(1:n)/n * sum(x))
  process <- sapply(1:nrow(S),function(s) 1/sqrt(n) * sqrt( S[s,] %*% solve(Omega) %*% as.matrix(S[s,])) )
  
  result <- abs(process)
  
  return(list(stat = max(result),cp = vec.est,proc = result))
}

block.boot <- function(data, B, vec_est = c(0,0,0,0,0,0,0,0), est=T){
  n <- dim(data)[1]
  d <- dim(data)[2]
  
  boot_dist <- rep(NA,B)
  
  dataDemean <- apply(data,2,function(x) x-mean(x))
  T <- length(data[,1])
  l <- floor(T^(1/3))
  anzbl <- floor(T/l)
  if(l * anzbl < T) anzbl <- anzbl+1
  
  if (est == T){
    vec_est <- apply(data,2,argmaxest)
  }
  
  if ((vec_est[1]>0) & (vec_est[2]>0) & (vec_est[3]>0) & (vec_est[4]>0) & (vec_est[5]>0) & (vec_est[6]>0) & (vec_est[7]>0) & (vec_est[8]>0)){
  sd1 <- rep(NA,d)
  sd2 <- rep(NA,d)
  
  for (jj in 1:d){
    sd1[jj] <- sd(data[1:vec_est[jj],jj])
    sd2[jj] <- sd(data[-(1:vec_est[jj]),jj])
  }
  }
  
  for(b in 1:B)
  {
    print(b)
    # set.seed(b)
    index <- sample(1:anzbl,anzbl,replace=T)
    index <- sapply(index,function(x) seq(1+(x-1)*l,x*l,by=1))
    index <- as.vector(index)
    problem <- which(index > T)
    if(length(problem) > 0) index[problem] <- index[problem]-T
    xboot <- dataDemean[index,]
    
    for (kk in 1:d)
    {
      if(vec_est[kk] > 0)
      {xboot[1:vec_est[kk],kk] <- xboot[1:vec_est[kk],kk] / sd(xboot[1:vec_est[kk],kk]) * sd1[kk]
      xboot[-(1:vec_est[kk]),kk] <- xboot[-(1:vec_est[kk]),kk] / sd(xboot[-(1:vec_est[kk]),kk]) * sd2[kk]
      }
      if(vec_est[kk] == 0)
      {xboot[,kk] <- xboot[,kk] / sd(xboot[,kk])}
    }
    boot_dist[b] = cusumtest(xboot,vec.est = vec_est, est = est)$stat
  }
return(boot_dist)  
}  

mv_corr_changes <- function(data){
  n <- dim(data)[1]
  d <- dim(data)[2]
  
  B <- 199 # # of bootstrap
  
  a0 <- .05  # This is the initial level
  
  locch <- c() # This is the vector of detected changes located
  locchsort <- c() # This is the ordered vector of changes
  tpch <- c() # This is the vector of the statistics associated with locch
  tpchsort <- c() # This are the statistics associated with locchsort
  
  look <- 1 # This is an indicator variable to check if the algorithm should look for new changes
  iter <- 0 # Iteration number
  
  while (look == 1){
  iter <- iter + 1
  llocch <- length(locch)
  locch2 <- c(0,locchsort,n)
  
  mtp <- htp <- ptp <- c()
  for (i in 1:(llocch+1)){
    nini <- locch2[i] + 1
    nend <- locch2[i+1]
    
    tp <- rep(0,n)
    vec_est <- apply(data,2,argmaxest)
    if ((nini < min(vec_est)) & (nend > max(vec_est))){
      cp1 <- 955-nini+1
      cp2 <- 964-nini+1
      cp3 <- 918-nini+1
      cp4 <- 957-nini+1
      cp5 <- 964-nini+1
      cp6 <- 952-nini+1
      cp7 <- 786-nini+1
      cp8 <- 784-nini+1
      cp <- c(cp1,cp2,cp3,cp4,cp5,cp6,cp7,cp8)
      tp[nini:nend] <- cusumtest(data[nini:nend,],vec.est = cp, est = F)$proc
      mtp[i] <- max(tp)
      htp[i] <- which.max(tp)
      boot_dist <- block.boot(data[nini:nend,],B,vec_est=cp,est=F)
      ptp[i] <- mean(mtp[i] <= boot_dist,na.rm = T)
    } else {
    tp[nini:nend] <- cusumtest(data[nini:nend,],vec.est = c(0,0,0,0,0,0,0,0),est=F)$proc
    mtp[i] <- max(tp)
    htp[i] <- which.max(tp)
    boot_dist <- block.boot(data[nini:nend,],B,vec_est=c(0,0,0,0,0,0,0,0),est=F)
    ptp[i] <- mean(mtp[i] <= boot_dist,na.rm = T)
    }
  }
  mmtp <- min(ptp)# Minimum of the p-values in this iteration
  hhtp <- htp[which.max(mtp)] # Estimation of the new changepoint (candidate)
  sl <- 1-(1 - a0)^(1/(length(locch)+1))
  
  if (mmtp < sl){
    if (any(abs(locch - hhtp) < (.05 * n))){
      look = 0
    }else{
      locch <- c(locch,hhtp)
      tpch <- c(tpch,mmtp)
    }   
    sortlocch <- sort(locch,index.return=TRUE)
    locchsort <- sortlocch$x
    tpchsort <- tpch[sortlocch$ix]
  }else{
    look <- 0
  }  
}
  locbeforeiteration <- locchsort
  datebeforeiteration <- mydate[locchsort+1]
  
#Refinement step
  
  if (length(locchsort) == 0){
    locchanges <- tpchanges <- c()
  }else if (length(locchsort) == 1){
    locchanges <- locchsort
    tpchanges <- tpchsort
  }else{
    ite <- 1
    while (ite == 1){   
      locchanges <- c(0,locchsort,n)
      lc <- length(locchanges) - 2
      locch <- hch <- pch <- c()
      vec_est <- apply(data,2,argmaxest)
      for (i in 1 : lc){
        tp <- rep(0,n)
        nini <- locchanges[i] + 1
        nend <- locchanges[i+2]
        
        if(nend < min(vec_est)|| nini>max(vec_est)){
          tp[nini:nend] <- cusumtest(data[nini:nend,],vec.est = c(0,0,0,0,0,0,0,0),est =F)$proc
          hch[i] <- max(tp)
          locch[i] <- which.max(tp)
          
          B <- 199
          boot_dist <- block.boot(data[nini:nend,],B,vec_est = c(0,0,0,0,0,0,0,0),est=F)
          pch[i] = mean(hch[i] <= boot_dist,na.rm=T)
        }
        if ((nend > max(vec_est)) & (nini < min(vec_est))){
          cp1 <- 955-nini+1
          cp2 <- 964-nini+1
          cp3 <- 918-nini+1
          cp4 <- 957-nini+1
          cp5 <- 964-nini+1
          cp6 <- 952-nini+1
          cp7 <- 786-nini+1
          cp8 <- 784-nini+1
          cp <- c(cp1,cp2,cp3,cp4,cp5,cp6,cp7,cp8)
          tp[nini:nend] <- cusumtest(data[nini:nend,],vec.est = cp, est=F)$proc
          hch[i] <- max(tp)
          locch[i] <- which.max(tp)
          
          B <- 199
          boot_dist <- block.boot(data[nini:nend,],B,vec_est = cp, est=F)
          pch[i] = mean(hch[i] <= boot_dist,na.rm=T)
        }
      }
      
      mhch <- max(pch)
      wmhch <- which.max(pch)
      if (mhch > a0){
        locchsort <- locchsort[-wmhch]
      }else{
        sortlocch <- sort(locch,index.return=TRUE)
        locchanges <- sortlocch$x
        tpchanges <- pch[sortlocch$ix]
        ite <- 0
      }
    }
  }      
  estchanges <- locchanges / n
  dateafteriteration <- mydate[locchanges+1]  

return(list("locchanges"=locchanges,"tpchanges"=tpchanges,"estchanges"=estchanges,"locbeforeiteration"=locbeforeiteration,"datebeforeiteration"=datebeforeiteration,"dateafteriteration"=dateafteriteration))
  
}
